@@ -107,9 +107,9 @@ let send_dirfull_warning dir full line1 =
     let send_mail_again =
       try
         let last = Hashtbl.find last_sent_dir_warning dir in
-	last < time_threshold
-      with Not_found -> true in
-
+        last < time_threshold
+      with Not_found -> true
+    in
     if send_mail_again then begin
       if full then Hashtbl.replace last_sent_dir_warning dir current_time;
       CommonEvent.add_event (Console_message_event
@@ -118,16 +118,20 @@ let send_dirfull_warning dir full line1 =
         let module M = Mailer in
         let subject = Printf.sprintf "[mldonkey@%s] AUTOMATED WARNING: %s %s" (Unix.gethostname ()) dir status in
         let mail = {
-          M.mail_to = !!mail; M.mail_from = !!mail;
-	  M.mail_subject = subject; M.mail_body = line1;
+          M.mail_to = !!mail;
+          M.mail_from = !!mail;
+          M.mail_subject = subject;
+          M.mail_body = line1;
+          M.smtp_login = !!smtp_login;
+          M.smtp_password = !!smtp_password;
         } in
-	try
+        try
           M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
-	with _ -> ()
+        with _ -> ()
     end
   end
 
-let file_commited_name incoming_dir file =
+let file_committed_name incoming_dir file =
   (try Unix2.safe_mkdir incoming_dir with _ -> ());
   let fs = Unix32.filesystem incoming_dir in
   let namemax =
@@ -151,7 +155,7 @@ let file_commited_name incoming_dir file =
       in
       iter 1
     else new_name in
-  set_file_best_name file (file_best_name file) "" 0;
+  set_file_best_name file (file_best_name file) fs namemax;
   Filename.concat incoming_dir new_name
 
 let script_for_file file incoming new_name =
@@ -240,7 +244,7 @@ let file_commit file =
             ()
           in
 
-	  let new_name = file_commited_name incoming.shdir_dirname file in
+	  let new_name = file_committed_name incoming.shdir_dirname file in
 	    if Unix2.is_directory file_name then begin
 	      Unix2.safe_mkdir new_name;
 	      Unix2.chmod new_name !Unix32.create_dir_mode;
@@ -268,7 +272,7 @@ let file_commit file =
                       incoming.shdir_dirname incoming.shdir_priority
                       best_name new_name);
               with e ->
-                  lprintf_nl "Exception %s while trying to share commited file"
+                  lprintf_nl "Exception %s while trying to share committed file"
                     (Printexc2.to_string e);
             end;
 
@@ -378,6 +382,8 @@ let mail_for_completed_file file =
         M.mail_from = address;
         M.mail_subject = subject;
         M.mail_body = line1 ^ line2 ^ line3 ^ line4 ^ line5 ^ (if admin then line6 else "") ^ line7;
+        M.smtp_login = !!smtp_login;
+        M.smtp_password = !!smtp_password;
       } in
         M.sendmail !!smtp_server !!smtp_port !!add_mail_brackets mail
     in
@@ -503,6 +509,7 @@ let contact_remove c =
       lprintf_nl "Exception in contact_remove: %s" (Printexc2.to_string e)
 
 let clean_exit n =
+  begin
   let can_exit = networks_for_all network_clean_exit in
   if can_exit then exit_properly n
   else 
@@ -512,7 +519,15 @@ let clean_exit n =
         if can_exit || retry_counter > !!shutdown_timeout then
 	  exit_properly n
 	else retry_later (retry_counter + 1)) in
-    retry_later 0
+    retry_later 0;
+
+  if (upnp_port_forwarding ()) then
+    begin
+    if !!clear_upnp_port_at_exit then
+      UpnpClient.remove_all_maps 0 ;
+    UpnpClient.job_stop 3;
+    end;
+  end
 
 let time_of_sec sec =
   let hours = sec / 60 / 60 in
