@@ -303,7 +303,7 @@ let results_ini = create_options_file "results.ini"
 let files_ini = create_options_file "files.ini"
 let friends_ini = create_options_file "friends.ini"
 
-let messages_log = ref "messages.log"
+let messages_log = "messages.log"
 
 let servers_section = file_section servers_ini [] ""
 
@@ -539,6 +539,15 @@ let alias_commands = define_option current_section ["alias_commands"]
       "exit", "q";
     ]
 
+let upnp_port_forwarding = define_option current_section ["upnp_port_forwarding"]
+  ~restart: true
+  "upnp port forwarding"
+    bool_option false
+
+let clear_upnp_port_at_exit = define_option current_section ["clear_upnp_port_at_exit"]
+  "clear all upnp port forwarding before mldonkey exit"
+    bool_option true
+
 let verbosity = define_expert_option current_section ["verbosity"]
   "A space-separated list of keywords. Each keyword triggers
   printing information on the corresponding messages:
@@ -1130,13 +1139,19 @@ let http_proxy_port = define_option current_section ["http_proxy_port"]
   "Port of HTTP proxy"
     port_option 8080
 
+let http_proxy_login = define_option current_section ["http_proxy_login"]
+  "HTTP proxy login (leave empty if proxy doesn't require authentication)"
+    string_option ""
+
+let http_proxy_password = define_option current_section ["http_proxy_password"]
+  "HTTP proxy password"
+    string_option ""
+
 let http_proxy_tcp = define_option current_section ["http_proxy_tcp"]
   "Direct TCP connections to HTTP proxy (the proxy should support CONNECT)"
     bool_option false
 
 
-
-
 (*************************************************************************)
 (*                                                                       *)
 (*                         Mail section                                  *)
@@ -1146,13 +1161,21 @@ let http_proxy_tcp = define_option current_section ["http_proxy_tcp"]
 let current_section = mail_section
 
 let smtp_server = define_option current_section ["smtp_server"]
-  "The mail server you want to use (must be SMTP). Use hostname or IP address"
+  (_s"The mail server you want to use (must be SMTP). Use hostname or IP address")
     string_option "127.0.0.1"
 
 let smtp_port = define_option current_section ["smtp_port"]
-  "The port to use on the mail server (default 25)"
+  (_s"The port to use on the mail server (default 25)")
     port_option 25
 
+let smtp_login = define_option current_section ["smtp_login"]
+  (_s"Login to use for SMTP authentication (leave empty to disable). LOGIN, PLAIN and CRAM-MD5 methods are supported")
+  string_option ""
+
+let smtp_password = define_option current_section ["smtp_password"]
+  (_s"Password to use for SMTP authentication")
+  string_option ""
+
 let mail = define_option current_section ["mail"]
   "Your e-mail if you want to receive mails when downloads are completed"
     string_option ""
@@ -1217,11 +1240,11 @@ let max_recover_gap = define_option current_section ["max_recover_zeroes_gap"]
 
 let file_completed_cmd = define_option current_section ["file_completed_cmd"]
   "A command that is called when a file is committed, does not work on MinGW.
-  Arguments are (kept for compatability):
+  Arguments are (kept for compatibility):
     $1 - temp file name, without path
     $2 - file size
     $3 - filename of the committed file
-  Also these environment variables can be used (prefered way):
+  Also these environment variables can be used (preferred way):
     $TEMPNAME  - temp file name, including path
     $FILEID    - same as $1
     $FILESIZE  - same as $2
@@ -1240,7 +1263,7 @@ let file_completed_cmd = define_option current_section ["file_completed_cmd"]
 let file_started_cmd = define_option current_section ["file_started_cmd"]
   "The command which is called when a download is started. Arguments
   are '-file <num>'
-  Also these environment variables can be used (prefered way):
+  Also these environment variables can be used (preferred way):
     $TEMPNAME  - temp file name, including path
     $FILEID    - same as $1
     $FILESIZE  - same as $2
@@ -2052,14 +2075,20 @@ let http_proxy_tcp_update _ =
 
 let _ =
   let proxy_update _ =
-    http_proxy :=
-    (match !!http_proxy_server with
-        "" -> None
-      | _  -> Some (!!http_proxy_server, !!http_proxy_port));
+    let auth = match !!http_proxy_login with
+    | "" -> None
+    | _ -> Some (!!http_proxy_login, !!http_proxy_password)
+    in
+    http_proxy := 
+      (match !!http_proxy_server with
+      | "" -> None
+      | _  -> Some (!!http_proxy_server, !!http_proxy_port, auth));
     http_proxy_tcp_update ()
   in
   option_hook http_proxy_server proxy_update;
   option_hook http_proxy_port proxy_update;
+  option_hook http_proxy_login proxy_update;
+  option_hook http_proxy_password proxy_update;
   option_hook http_proxy_tcp http_proxy_tcp_update
 
 let _ =
