@@ -151,7 +151,7 @@ let mldonkey_wget_url w f =
       )
       (fun c ->
 	match c with
-          | x when x < 200 || x > 299 -> begin
+          | x when x > 0 && (x < 200 || x > 299) -> begin
             (* use local version if wget fail and file exists *)
             let file = Filename.concat "web_infos" (Filename.basename r.H.req_url.Url.short_file) in
             (* mark this job downloaded *)
