@@ -181,7 +181,7 @@ let op_file_commit file new_name =
               file.file_torrent_diskname new_torrent_diskname));
       file.file_torrent_diskname <- new_torrent_diskname;
 
-      (* update file_shared with new path to commited file *)
+      (* update file_shared with new path to committed file *)
       match file.file_shared with
       | None -> ()
       | Some old_impl ->
@@ -234,14 +234,15 @@ let op_file_print file o =
   Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
   let tracker_header_printed = ref false in
   List.iter (fun tracker ->
+    let tracker_url = show_tracker_url tracker.tracker_url in
     let tracker_text =
       match tracker.tracker_status with
         | Disabled s | Disabled_mld s ->
-            Printf.sprintf "\\<font color=\\\"red\\\"\\>disabled: %s\\<br\\>\\--error: %s\\</font\\>" tracker.tracker_url s
+            Printf.sprintf "\\<font color=\\\"red\\\"\\>disabled: %s\\<br\\>\\--error: %s\\</font\\>" tracker_url s
         | Disabled_failure (i,s) -> 
-            Printf.sprintf "\\<font color=\\\"red\\\"\\>disabled: %s\\<br\\>\\--error: %s (try %d)\\</font\\>" tracker.tracker_url s i
+            Printf.sprintf "\\<font color=\\\"red\\\"\\>disabled: %s\\<br\\>\\--error: %s (try %d)\\</font\\>" tracker_url s i
         | _ ->
-            Printf.sprintf "enabled: %s" tracker.tracker_url
+            Printf.sprintf "enabled: %s" tracker_url
 
     in
     html_mods_td buf [
@@ -250,7 +251,7 @@ let op_file_print file o =
        else
         ("", "sr br", "")
       );
-      (tracker.tracker_url, "sr", tracker_text)];
+      (tracker_url, "sr", tracker_text)];
     Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
     tracker_header_printed := true;
   ) file.file_trackers;
@@ -421,12 +422,13 @@ let op_file_print file o =
 
   Printf.bprintf buf "Trackers:\n";
   List.iter (fun tracker ->
+    let tracker_url = show_tracker_url tracker.tracker_url in
     match tracker.tracker_status with
     | Disabled s | Disabled_mld s ->
-        Printf.bprintf buf "%s, disabled: %s\n" tracker.tracker_url s
+        Printf.bprintf buf "%s, disabled: %s\n" tracker_url s
     | Disabled_failure (i,s) -> 
-        Printf.bprintf buf "%s, disabled (try %d): %s\n" tracker.tracker_url i s
-    | _ -> Printf.bprintf buf "%s\n" tracker.tracker_url
+        Printf.bprintf buf "%s, disabled (try %d): %s\n" tracker_url i s
+    | _ -> Printf.bprintf buf "%s\n" tracker_url
   ) file.file_trackers;
   if file.file_torrent_diskname <> "" then
     Printf.bprintf buf "Torrent diskname: %s\n" file.file_torrent_diskname;
@@ -482,42 +484,42 @@ let op_file_print_sources file o =
   if Hashtbl.length file.file_clients > 0 then begin
 
       let header_list = [
-        ( "1", "srh br ac", "Client number", "Num" ) ;
-        ( "0", "srh br", "Client UID", "UID" ) ;
-        ( "0", "srh br", "Client software", "Soft" ) ;
-        ( "0", "srh", "IP address", "IP address" ) ;
-        ( "0", "srh br ar", "Port", "Port" ) ;
-        ] @ (if Geoip.active () then [( "0", "srh br ar", "Country Code/Name", "CC" )] else []) @ [
-        ( "1", "srh ar", "Total UL bytes to this client for all files", "tUL" ) ;
-        ( "1", "srh ar br", "Total DL bytes from this client for all files", "tDL" ) ;
-        ( "1", "srh ar", "Session UL bytes to this client for all files", "sUL" ) ;
-        ( "1", "srh ar br", "Session DL bytes from this client for all files", "sDL" ) ;
-        ( "0", "srh ar", "Interested [T]rue, [F]alse", "I" ) ;
-        ( "0", "srh ar", "Choked [T]rue, [F]alse", "C" ) ;
-        ( "1", "srh br ar", "Allowed to write", "A" ) ;
-        ( "0", "srh ar", "Interesting [T]rue, [F]alse", "I" );
-        ( "0", "srh ar", "Already sent interested [T]rue, [F]alse", "A" );
-        ( "0", "srh br ar", "Already sent not interested [T]rue, [F]alse", "N" );
+        ( Num, "srh br ac", "Client number", "Num" ) ;
+        ( Str, "srh br", "Client UID", "UID" ) ;
+        ( Str, "srh br", "Client software", "Soft" ) ;
+        ( Str, "srh", "IP address", "IP address" ) ;
+        ( Num, "srh br ar", "Port", "Port" ) ;
+        ] @ (if Geoip.active () then [( Str, "srh br ar", "Country Code/Name", "CC" )] else []) @ [
+        ( Num, "srh ar", "Total UL bytes to this client for all files", "tUL" ) ;
+        ( Num, "srh ar br", "Total DL bytes from this client for all files", "tDL" ) ;
+        ( Num, "srh ar", "Session UL bytes to this client for all files", "sUL" ) ;
+        ( Num, "srh ar br", "Session DL bytes from this client for all files", "sDL" ) ;
+        ( Str, "srh ar", "Interested [T]rue, [F]alse", "I" ) ;
+        ( Str, "srh ar", "Choked [T]rue, [F]alse", "C" ) ;
+        ( Num, "srh br ar", "Allowed to write", "A" ) ;
+        ( Str, "srh ar", "Interesting [T]rue, [F]alse", "I" );
+        ( Str, "srh ar", "Already sent interested [T]rue, [F]alse", "A" );
+        ( Str, "srh br ar", "Already sent not interested [T]rue, [F]alse", "N" );
 
-        ( "0", "srh ar", "Good [T]rue, [F]alse", "G" );
-        ( "0", "srh ar", "Incoming [T]rue, [F]alse", "I" );
-        ( "0", "srh br ar", "Registered bitfield [T]rue, [F]alse", "B" );
+        ( Str, "srh ar", "Good [T]rue, [F]alse", "G" );
+        ( Str, "srh ar", "Incoming [T]rue, [F]alse", "I" );
+        ( Str, "srh br ar", "Registered bitfield [T]rue, [F]alse", "B" );
 
-        ( "0", "srh ar", "Connect Time", "T" );
-        ( "0", "srh ar", "Last optimist", "L.Opt" );
-        ( "0", "srh br ar", "Num try", "N" );
+        ( Num, "srh ar", "Connection Time", "T" );
+        ( Str, "srh ar", "Last optimistic unchoke", "L.Opt" );
+        ( Num, "srh br ar", "Number of tries", "N" );
 
-        ( "0", "srh", "DHT [T]rue, [F]alse", "D" );
-        ( "0", "srh", "Cache extensions [T]rue, [F]alse", "C" );
-        ( "0", "srh", "Fast extensions [T]rue, [F]alse", "F" );
-        ( "0", "srh", "uTorrent extensions [T]rue, [F]alse", "U" );
-        ( "0", "srh br", "Azureus messaging protocol [T]rue, [F]alse", "A" );
+        ( Str, "srh", "DHT [T]rue, [F]alse", "D" );
+        ( Str, "srh", "Cache extensions [T]rue, [F]alse", "C" );
+        ( Str, "srh", "Fast extensions [T]rue, [F]alse", "F" );
+        ( Str, "srh", "uTorrent extensions [T]rue, [F]alse", "U" );
+        ( Str, "srh br", "Azureus messaging protocol [T]rue, [F]alse", "A" );
 (*
-        ( "0", "srh", "Bitmap (absent|partial|present|verified)", (colored_chunks
+        ( Str, "srh", "Bitmap (absent|partial|present|verified)", (colored_chunks
         (Array.init (String.length info.G.file_chunks)
         (fun i -> ((int_of_char info.G.file_chunks.[i])-48)))) ) ;
 *)
-        ( "1", "srh ar", "Number of full chunks", (Printf.sprintf "%d"
+        ( Num, "srh ar", "Number of full chunks", (Printf.sprintf "%d"
           (match file.file_swarmer with
           | None -> 0
           | Some swarmer ->
@@ -564,7 +566,7 @@ let op_file_print_sources file o =
             ("", "sr", (btos c.client_incoming));
             ("", "br sr", (btos c.client_registered_bitfield));
 
-            ("", "sr", Printf.sprintf "%d" c.client_connect_time);
+            ("", "sr", Printf.sprintf "%d" ((last_time () - c.client_connect_time) / 60));
             ("", "ar sr", string_of_date c.client_last_optimist);
             ("", "br sr", Printf.sprintf "%d" c.client_num_try);
 
@@ -697,22 +699,11 @@ let load_torrent_string s user group =
   (* Save the torrent, because we later want to put
      it in the seeded directory. *)
   let torrent_is_usable = ref false in
-  let can_handle_tracker url =
-    String2.check_prefix url "http://" in
-  List.iter (fun url -> if can_handle_tracker url then torrent_is_usable := true)
+  List.iter (fun url -> if can_handle_tracker (make_tracker_url url) then torrent_is_usable := true)
     (if torrent.torrent_announce_list <> [] then torrent.torrent_announce_list else [torrent.torrent_announce]);
   if not !torrent_is_usable then raise (Torrent_can_not_be_used torrent.torrent_name);
 
-  let torrent_diskname =
-    let fs = Unix32.filesystem downloads_directory in
-    let namemax =
-      match Unix32.fnamelen downloads_directory with
-      | None -> 0
-      | Some v -> v
-    in
-    Filename.concat downloads_directory
-    (Filename2.filesystem_compliant torrent.torrent_name fs namemax) ^ ".torrent"
-    in
+  let torrent_diskname = CommonFile.concat_file downloads_directory (torrent.torrent_name ^ ".torrent") in
   if Sys.file_exists torrent_diskname then
     begin
       if !verbose then lprintf_nl "load_torrent_string: %s already exists, ignoring" torrent_diskname;
@@ -1190,8 +1181,8 @@ let commands =
         if use_html_mods o then begin
           html_mods_cntr_init ();
           html_mods_table_header o.conn_buf "sourcesInfo" "sourcesInfo" [
-            ( "0", "srh br", "File Info", "Info" ) ;
-            ( "0", "srh", "Value", "Value" ) ]
+            ( Str, "srh br", "File Info", "Info" ) ;
+            ( Str, "srh", "Value", "Value" ) ]
         end;
         op_file_print file.impl_file_val o;
         if use_html_mods o then begin
