@@ -145,7 +145,7 @@ let decode_torrent s =
             match key, value with
             | "announce", String tracker_url ->
                if !verbose_msg_servers then
-                 lprintf_nl "[BT] New tracker added :%s" tracker_url;
+                 lprintf_nl "[BT] New tracker added: %s" tracker_url;
                 announce := tracker_url
             | "announce-list", List list ->
                 List.iter (fun url_list ->
@@ -280,9 +280,9 @@ let decode_torrent s =
   assert (real_file_name <> "");
   assert (!file_piece_size <> zero);
   assert (!file_pieces <> "");
-  assert (!file_info = Bencode.decode (Bencode.encode !file_info));
+  assert (!file_info = Bencode.decode (Bencode.encode ~strict:false !file_info));
 
-  let file_id = Sha1.string (Bencode.encode !file_info) in
+  let file_id = Sha1.string (Bencode.encode ~strict:false !file_info) in
   let npieces = 1 + Int64.to_int ((!length -- one) // !file_piece_size) in
   let pieces = Array.init npieces (fun i ->
         let s = String.sub !file_pieces (i*20) 20 in
@@ -387,7 +387,7 @@ let encode_torrent torrent =
 
 let make_torrent announce filename comment is_private =
   let announce_list = [ announce ] in
-  let basename = Filename.basename filename in
+  let basename = Filename2.basename filename in
   let files, t =
     if Unix2.is_directory filename then
       let rec iter_directory list dirname =
